from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.test.client import RequestFactory
from django.urls import resolve
from django.urls.exceptions import Resolver404
from django.utils.module_loading import import_string
from urllib.parse import urljoin

factory = RequestFactory()

def get_exceptions():
    return [
        import_string(e) if isinstance(e, str) else e
        for e in getattr(settings, 'PAGE_VISIBILITY_EXCEPTIONS', (PermissionDenied, Http404))
    ]
# Note - except clauses only take tuples of exceptions, not lists
EXCEPTIONS = tuple(get_exceptions())

class PageVisibilityError(Exception):
    '''
        Base class for all of the exceptions we raise
    '''
    pass
class ViewDoesNotSupportVisibilityTest(PageVisibilityError):
    pass
class LinkPathDoesNotExist(PageVisibilityError):
    pass

def is_visible_to_user(path, user, raise_exceptions=settings.DEBUG):
    '''
        Checks whether the given user is allowed to make a GET request to the given path.
    '''

    try :
        func, args, kwargs = resolve(path)
    except Resolver404 :
        if not raise_exceptions :
            return False
        raise LinkPathDoesNotExist(f'Link path {path} does not exist')

    '''
        Notice - this funtion is meant to work with special views that have an added "test_page_visibility" attribute.

        This attribute should be a function which takes the same parameters as the view itself, and returns a truthy value if the user can access the page.

        If the user is not allowed to access the page, the test function may either return a falsey value, or raise an exception.
    '''
    try :
        visibility_test = func.test_page_visibility
    except AttributeError :
        if not raise_exceptions :
            return False
        raise ViewDoesNotSupportVisibilityTest(f'Path {path} resolved to view function {func} which does implement test_page_visibility()')

    request = factory.get(path)
    request.user = user

    try :
        return visibility_test(request, *args, **kwargs) != False
    except EXCEPTIONS :
        return False
    return True

def is_visible(path, request, *args, **kwargs):
    '''
        Convenience wrapper around is_visible_to_user, which will cover most use cases
    '''

    # Resolve path relative to current request - allows you to pass in relative paths
    path = urljoin(request.get_full_path(), path)

    return is_visible_to_user(path, request.user, *args, **kwargs)



from django import template

from django_page_visibility import is_visible_to_user

register = template.Library()

class PermittedLinkNode(template.Node):
    '''
        Node for use with permitted_link tag
    '''
    DELETE = object()
    def __init__(self, path, name, nodelist):
        self.path = path
        self.name = name
        self.nodelist = nodelist
    def render(self, context):
        path = self.path.resolve(context)
        name = self.name
        if not is_visible_to_user(path, context.request.user) :
            return ''

        original_value = context.get(name, self.DELETE)
        context[name] = path
        ret = self.nodelist.render(context)

        if original_value == self.DELETE :
            del(context[name])
        else :
            context[name] = original_value
        return ret
@register.tag
def permitted_link(parser, token):
    '''
        Checks if the passed url path is visible to the current user.
        If it is, the contents of the tag are rendered, and the url path is put in context as "link".
        If not, contents are not rendered.

        usage:

        {% load page_visibility %}
        {% permitted_link "/protected_page/" %}
            <a href="{{link}}">Protected Page</a>
        {% end_permitted_link %}

        or:

        {% load page_visibility %}
        {% permitted_link "/protected_page/" as some_other_name %}
            <a href="{{some_other_name}}">Fake Permission Test</a>
        {% end_permitted_link %}
    '''
    bits = token.split_contents()
    if len(bits) == 2 :
        path = bits[1]
        name = 'link'
    elif len(bits) == 4 and bits[2] == 'as' :
        path = bits[1]
        name = bits[3]
    else :
        raise template.TemplateSyntaxError(
            f'{bits[0]} requires a single argument, or {{% {bits[0]}  "path" as my_link %}} syntax'
        )
    path = template.Variable(path)
    nodelist = parser.parse(('end_permitted_link',))
    parser.delete_first_token()
    return PermittedLinkNode(path, name, nodelist)

@register.filter
def is_permitted_to_see(user, path):
    '''
        usage:

        {% load page_visibility %}
        {% url 'protected_page' as path %}
        {% if request.user|is_permitted_to_see:path %}
            <a href="{{path}}">Protected Page</a>
        {% endif %}
    '''
    return is_visible_to_user(path, user)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("VERSION", "r") as fh:
    version = fh.read()

setuptools.setup(
    name="django_page_visibility",
    version=version,
    author="Alex Fischer",
    author_email="alex@quadrant.net",
    description="Implements a function to test whether a given url is currently visible to a given user.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/quadrant-newmedia/django_page_visibility",
    packages=["django_page_visibility"],
    package_dir={"": "src"},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=["Django>=3,<6"],
)
